#
#    Copyright (C) 2003-2004  James Tauber
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
This module abstracts the access to file contents and file attributes.

Alternative backend data stores such as databases or version control systems
could be implemented by implementing an identical interface.

This implementation works by storing each "file" as a directory with the
extension ".ldv". The directory may then contain not only the content
(potentially in multiple forms) but additional metadata, annotations, etc.
"""

import os        
        
        
class LeonardoFileSystem:
        

    def __init__(self, root):
        self.root = root

        # check access to filesystem root
        if not os.access(self.root, os.R_OK):
            # @@@ improve response to user
            raise "can't access filesystem root"
        

    def get(self, key):
        return LeonardoFile(self, key)


    def get_children(self, key):
        """
        Returns a tuple of:
          - list of directories that are children of the key
          - list of leonardo pages that are children of the key
        """

        directories = []
        files = []
    
        # @@@ factor out commonalities with LeonardoFile
        d = self.root + key.strip("/")

        if os.path.isdir(d) and not d.endswith(".ldv"):
            for f in os.listdir(d):
                # @@@ factor out ignore list
                if os.path.isdir(d + "/" + f) and f not in [".svn"]:
                    if f.endswith(".ldv"):
                        # @@@ change to creation time
                        sort_key = os.path.getmtime(os.path.join(d, f))
                        
                        files.append((sort_key, f[:-4]))
                    else:
                        directories.append(f)
        directories.sort()
        directories.reverse()
        files.sort()
        files.reverse()
        files = [c[1] for c in files]
            
        return directories, files



class Bundle:


    def get_content_filename_(self, content_type):
        return self.get_directory_() + "__content__." + content_type


    def get_content_filename_glob_(self):
        return self.get_content_filename_("*")


    def get_content_files_(self):
        import glob
        return glob.glob(self.get_content_filename_glob_())


    def get_property_filename_(self, name):
        return self.get_directory_() + name
        

    def exists(self):
        files = self.get_content_files_()
        if files:
            return True
        else:
            return False
        

    def load_content_(self):
        files = self.get_content_files_()
        if files:
            f = file(files[0])    
            s = f.read()
            f.close()
            self.content = s


    def load_content_type_(self):
        files = self.get_content_files_()
        if files:
            self.content_type = files[0].split(".")[-1]
        else:
            self.content_type = None


    def get_content(self):
        if not self.content:
            self.load_content_()
        return self.content        


    def get_content_type(self):
        if not self.content_type:
            self.load_content_type_()
        return self.content_type
    

    def get_lastmod(self):
        import os
        files = self.get_content_files_()
        ST_MTIME = 8
        if files:
            return os.stat(files[0])[ST_MTIME]
        else:
            return -1


    def get_creation_time(self):
        t = self.get_property("creation_time", "-1")
        return int(t)


    def set_content(self, content, content_type):
        self.make_bundle_()
        self.content = content
        self.content_type = content_type
        path = self.get_content_filename_(content_type)
        f = file(path, "w")
        f.write(content)
        f.close()


    def delete(self):
        from os.path import join
        for root, dirs, files in os.walk(self.get_directory_(), topdown=False):
            for name in files:
                os.remove(join(root, name))
            for name in dirs:
                os.rmdir(join(root, name))
        os.rmdir(self.get_directory_())
        

    def make_bundle_(self):
        directory = self.get_directory_()
        if not os.access(directory, os.F_OK):
            import time
            os.makedirs(directory)
            self.set_property("creation_time", time.time())


    def get_property(self, name, default=None):
        filename = self.get_property_filename_(name)
        if os.access(filename, os.R_OK):
            f = file(self.get_property_filename_(name))    
            s = f.read()
            f.close()
            return s
        else:
            return default
        
    def set_property(self, name, value):
        self.make_bundle_()
        path = self.get_property_filename_(name)
        f = file(path, "w")
        f.write(str(value))
        f.close()



class LeonardoFile(Bundle):

    
    def __init__(self, filesystem, key):
        self.filesystem = filesystem
        self.key = key
        self.content = None
        self.content_type = None


    def get_directory_(self):
        return self.filesystem.root + self.key.strip("/") + ".ldv/"


    def enclosure(self, enctype, index):
        return Enclosure(self, enctype, index)
        
        
    def enclosures(self, enctype=None):
        pass # @@@ TODO
        


class Enclosure(Bundle):

    
    def __init__(self, parent, enctype, index):
        self.parent = parent
        self.enctype = enctype
        self.index = index
        self.content = None
        self.content_type = None


    def get_directory_(self):
        return self.parent.filesystem.root + self.parent.key.strip("/") + ".ldv/" + "__" + self.enctype + "__" + str(self.index) + "/"
